@@ -1,6 +1,6 @@
 import asyncio
 from collections import UserList
-from typing import Union
+from typing import Union, Tuple
 
 from astropy.time import Time
 from pandas import DataFrame
@@ -66,8 +66,17 @@ class TessImageList(UserList):
         obj._catalog = catalog
         return obj
 
-    async def _get_cutouts(self, crdlist: TessCoordList, shape):
-        async with self[0]._get_default_client() as client:
+    def _get_default_client(self):
+        for img in self:
+            if img.url:
+                return img._get_default_client()
+        # If all images are None, the client type doesn't matter
+        return img._get_default_client(client_type="s3")
+
+    async def _get_cutouts(self, crdlist: TessCoordList, shape: Tuple[int, int]):
+        if len(self) == 0:
+            return []
+        async with self._get_default_client() as client:
             # Create list of functions to be executed
             flist = [
                 img.async_cutout(
@@ -89,27 +98,25 @@ class TessImageList(UserList):
             results = [t.result() for t in tasks]
             return results
 
-    def cutout(self, column: int, row: int, shape=(5, 5)):
+    def cutout(self, column: int, row: int, shape: Tuple[int, int] = (5, 5)):
         # Turn (column, row) into a TessCoordList to match the interface of _get_cutouts
         crdlist = TessCoordList(
             [
                 TessCoord(
-                    sector=self[0].sector,
-                    camera=self[0].camera,
-                    ccd=self[0].ccd,
+                    sector=img.sector if img.url else 1,  # TessCoord needs valid number
+                    camera=img.camera if img.url else 1,
+                    ccd=img.ccd if img.url else 1,
                     column=column,
                     row=row,
                 )
+                for img in self
             ]
-            * len(self)
         )
         return self.moving_cutout(crdlist=crdlist, shape=shape)
 
-    def moving_cutout(self, crdlist: TessCoordList, shape=(5, 5)):
+    def moving_cutout(self, crdlist: TessCoordList, shape: Tuple[int, int] = (5, 5)):
         cutouts = asyncio.run(self._get_cutouts(crdlist=crdlist, shape=shape))
-        tpf = TargetPixelFile.from_cutouts(
-            cutouts  # , extra_columns=["sector", "camera", "ccd"]
-        )
+        tpf = TargetPixelFile.from_cutouts(cutouts)
         return tpf.to_lightkurve()
 
 
@@ -125,7 +132,9 @@ def list_images(
     if author == "tica":
         from . import tica
 
-        return tica.list_tica_images(sector=sector, camera=camera, ccd=ccd)
+        return tica.list_tica_images(
+            sector=sector, camera=camera, ccd=ccd, time=time, provider=provider
+        )
     else:
         from . import spoc
 
