@@ -16,6 +16,8 @@ ColDefs(
 )
 """
 from datetime import datetime
+from typing import Optional
+
 import numpy as np
 from numpy import array, ndarray
 
@@ -32,38 +34,56 @@ TPF_OPTIONAL_COLUMNS = {
     "SECTOR": {"format": "I"},
     "CAMERA": {"format": "B"},
     "CCD": {"format": "B"},
-    "COLUMN": {"format": "I"},
-    "ROW": {"format": "I"},
+    "CORNER_COLUMN": {"format": "I", "unit": "pixels"},
+    "CORNER_ROW": {"format": "I", "unit": "pixels"},
+    "TARGET_COLUMN": {"format": "E", "unit": "pixels"},
+    "TARGET_ROW": {"format": "E", "unit": "pixels"},
+    "URL": {"format": "A150"},
 }
 
 
 class TargetPixelFile:
     """Class representation of a Target Pixel File (TPF)."""
 
-    _optional_column_data = {}
-
     def __init__(
         self,
         time: ndarray,
         flux: ndarray,
         flux_err: ndarray,
-        wcs: WCS = None,
-        meta: dict = None,
+        flux_bkg: Optional[ndarray] = None,
+        flux_bkg_err: Optional[ndarray] = None,
+        wcs: Optional[WCS] = None,
+        meta: Optional[dict] = None,
     ):
         self.time = time
         self.flux = flux
         self.flux_err = flux_err
+        self.flux_bkg = flux_bkg
+        if self.flux_bkg is None:
+            self.flux_bkg = np.empty(shape=flux.shape)
+            self.flux_bkg[:] = np.nan
+        self.flux_bkg_err = flux_bkg_err
+        if self.flux_bkg_err is None:
+            self.flux_bkg_err = np.empty(shape=flux.shape)
+            self.flux_bkg_err[:] = np.nan
         self.wcs = wcs
-        if meta is None:
-            meta = {}
-            meta["TELESCOP"] = "TESS"
-            meta["ORIGIN"] = "tess_cloud"
-            meta["CREATOR"] = "tess_cloud.targetpixelfile"
-            meta["DATE"] = datetime.now().strftime("%Y-%m-%d")
-        self.meta = meta
 
-    def add_column(self, name, array):
+        meta_default = {
+            "TELESCOP": "TESS",
+            "ORIGIN": "tess_cloud",
+            "CREATOR": "tess_cloud.targetpixelfile",
+            "DATE": datetime.now().strftime("%Y-%m-%d"),
+        }
+        if meta:
+            meta_default.update(meta)
+        self.meta = meta_default
+
+        self._optional_column_data = {}
+
+    def add_column(self, name, array, colspec=None):
         self._optional_column_data[name] = array
+        if colspec:
+            TPF_OPTIONAL_COLUMNS[name] = colspec
 
     @property
     def n_cadences(self):
@@ -106,7 +126,10 @@ class TargetPixelFile:
 
     @staticmethod
     def from_cutouts(images: list) -> "TargetPixelFile":
-        shape = (len(images), images[0].flux.shape[0], images[0].flux.shape[1])
+        if len(images) > 0:
+            shape = (len(images), images[0].flux.shape[0], images[0].flux.shape[1])
+        else:
+            shape = (0, 0, 0)
         flux = np.empty(shape)
         flux_err = np.empty(shape)
         for idx, img in enumerate(images):
@@ -127,9 +150,19 @@ class TargetPixelFile:
         tpf.add_column(name="SECTOR", array=np.array([img.sector for img in images]))
         tpf.add_column(name="CAMERA", array=np.array([img.camera for img in images]))
         tpf.add_column(name="CCD", array=np.array([img.ccd for img in images]))
-        tpf.add_column(name="COLUMN", array=np.array([img.column for img in images]))
-        tpf.add_column(name="ROW", array=np.array([img.row for img in images]))
-
+        tpf.add_column(
+            name="CORNER_COLUMN", array=np.array([img.corner_column for img in images])
+        )
+        tpf.add_column(
+            name="CORNER_ROW", array=np.array([img.corner_row for img in images])
+        )
+        tpf.add_column(
+            name="TARGET_COLUMN", array=np.array([img.target_column for img in images])
+        )
+        tpf.add_column(
+            name="TARGET_ROW", array=np.array([img.target_row for img in images])
+        )
+        tpf.add_column(name="URL", array=np.array([img.url for img in images]))
         return tpf
 
     @staticmethod
@@ -148,8 +181,10 @@ class TargetPixelFile:
         hdulist = self._create_hdulist()
         return hdulist.writeto(*args, **kwargs)
 
-    def to_lightkurve(self):
-        return lk.TessTargetPixelFile(self._create_hdulist())
+    def to_lightkurve(self, quality_bitmask=0):
+        return lk.TessTargetPixelFile(
+            self._create_hdulist(), quality_bitmask=quality_bitmask
+        )
 
     def _create_hdulist(self):
         """Returns an astropy.io.fits.HDUList object."""
@@ -167,15 +202,21 @@ class TargetPixelFile:
         hdu.header.update(self.meta)
         return hdu
 
+    @property
+    def _coldim(self):
+        return "({},{})".format(self.n_columns, self.n_rows)
+
+    @property
+    def _eformat(self):
+        return "{}E".format(self.n_rows * self.n_columns)
+
     def _create_table_extension(self):
         """Create the 'TARGETTABLES' extension (i.e. extension #1)."""
         # Turn the data arrays into fits columns and initialize the HDU
-        coldim = "({},{})".format(self.n_columns, self.n_rows)
-        eformat = "{}E".format(self.n_rows * self.n_columns)
         jformat = "{}J".format(self.n_rows * self.n_columns)
         cols = []
         cols.append(
-            fits.Column(name="TIME", format="D", unit="BJD - 2454833", array=self.time)
+            fits.Column(name="TIME", format="D", unit="BJD - 2457000", array=self.time)
         )
         cols.append(
             fits.Column(name="TIMECORR", format="E", unit="D", array=self.timecorr)
@@ -185,24 +226,46 @@ class TargetPixelFile:
                 name="RAW_CNTS",
                 format=jformat,
                 unit="count",
-                dim=coldim,
+                dim=self._coldim,
                 array=self.raw_cnts,
             )
         )
         cols.append(
             fits.Column(
-                name="FLUX", format=eformat, unit="e-/s", dim=coldim, array=self.flux
+                name="FLUX",
+                format=self._eformat,
+                unit="e-/s",
+                dim=self._coldim,
+                array=self.flux,
             )
         )
         cols.append(
             fits.Column(
                 name="FLUX_ERR",
-                format=eformat,
+                format=self._eformat,
                 unit="e-/s",
-                dim=coldim,
+                dim=self._coldim,
                 array=self.flux_err,
             )
         )
+        cols.append(
+            fits.Column(
+                name="FLUX_BKG",
+                format=self._eformat,
+                unit="e-/s",
+                dim=self._coldim,
+                array=self.flux_bkg,
+            )
+        )
+        cols.append(
+            fits.Column(
+                name="FLUX_BKG_ERR",
+                format=self._eformat,
+                unit="e-/s",
+                dim=self._coldim,
+                array=self.flux_bkg_err,
+            )
+        )
 
         for name in self._optional_column_data:
             key = name.upper()
@@ -212,11 +275,13 @@ class TargetPixelFile:
                     array=self._optional_column_data[key],
                     format=TPF_OPTIONAL_COLUMNS[key].get("format", ""),
                     unit=TPF_OPTIONAL_COLUMNS[key].get("unit", ""),
+                    dim=TPF_OPTIONAL_COLUMNS[key].get("dim", ""),
                 )
             )
 
         coldefs = fits.ColDefs(cols)
         hdu = fits.BinTableHDU.from_columns(coldefs)
+        hdu.header["BJDREFI"] = 2457000
 
         return hdu
 
